import logging
import pytest
import sys
from bandit.core.config import BanditConfig
from bandit.core.manager import BanditManager
from bandit.core import constants
from pytest_bandit.errors import BanditError

bandit_config_params = []

LOG = logging.getLogger(__name__)


class BanditItem(pytest.Item):
    CACHE_KEY = 'bandit/mtimes'

    def __init__(self, session):
        super().__init__('bandit', session)
        self.add_marker('bandit')
        self.config = session.config

    def setup(self):
        old_mtime = self.config.cache.get(self.CACHE_KEY, {}).get(str(self.fspath), -1)
        mtime = self.fspath.mtime()
        if old_mtime == mtime:
            pytest.skip('previously passed bandit checks')

    def runtest(self):
        b_conf = BanditConfig()
        b_mgr = BanditManager(b_conf,
                              self.config.getini('bandit_aggregate_by'),
                              self.config.getini('bandit_debug'),
                              profile=self.config.getini('bandit_profile'),
                              verbose=self.config.getini('bandit_verbose'),
                              ignore_nosec=self.config.getini('bandit_ignore_nosec'))
        b_mgr.discover_files(self.config.getini('bandit_targets'),
                             self.config.getini('bandit_recurse'),
                             self.config.getini('bandit_exclude'))
        b_mgr.run_tests()

        LOG.debug(b_mgr.b_ma)
        LOG.debug(b_mgr.metrics)

        # trigger output of results by Bandit Manager
        sev_level = constants.RANKING[self.config.getini('bandit_sev_level') - 1]
        conf_level = constants.RANKING[self.config.getini('bandit_conf_level') - 1]
        sys.stdout = sys.__stdout__
        b_mgr.output_results(self.config.getini('bandit_context_lines'),
                             sev_level,
                             conf_level,
                             sys.stdout,
                             'screen')

        # return an exit code of 1 if there are results, 0 otherwise
        if b_mgr.results_count(sev_filter=sev_level, conf_filter=conf_level) > 0:
            return 1
        else:
            return 0

    def repr_failure(self, excinfo):
        if excinfo.errisinstance(BanditError):
            return excinfo.value.args[0]
        else:
            return super().repr_failure(excinfo)

    def reportinfo(self):
        return self.fspath, None, 'bandit-check'



# -*- coding: utf-8 -*-
import bandit
import py.io
import pytest
import sys

from pytest_bandit.controller import BanditItem
from pytest_bandit.errors import BanditError


def pytest_addoption(parser):
    group = parser.getgroup('bandit')
    group.addoption(
        '--bandit',
        action='store_true',
        default=False,
        dest='run_bandit',
        help='Run bandit'
    )
    parser.addini(
        'bandit_targets',
        type='args',
        help='Path to analyzed code'
    )
    parser.addini(
        'bandit_recurse',
        type='bool',
        default=True,
        help='Path to analyzed code'
    )
    parser.addini(
        'bandit_aggregate_by',
        type='args',
        default='vuln',
        help='Aggregate output by vulnerability (default) or by filename'
    )
    parser.addini(
        'bandit_context_lines',
        type='args',
        default=3,
        help='Maximum number of code lines to output for each issue'
    )
    parser.addini(
        'bandit_config',
        type='args',
        help='Optional config file to use for selecting plugins and overriding defaults'
    )
    parser.addini(
        'bandit_profile',
        type='args',
        default=None,
        help='Profile to use (defaults to executing all tests)'
    )
    parser.addini(
        'bandit_run_tests',
        type='args',
        help='Comma-separated list of test IDs to run'
    )
    parser.addini(
        'bandit_skip_tests',
        type='args',
        help='Comma-separated list of test IDs to skip'
    )
    parser.addini(
        'bandit_sev_level',
        type='args',
        default=1,
        help='Report only issues of a given severity level or higher (1 = LOW, 2 = MEDIUM, 3 = HIGH)'
    )
    parser.addini(
        'bandit_conf_level',
        type='args',
        default=1,
        help='Report only issues of a given confidence level or higher (1 = LOW, 2 = MEDIUM, 3 = HIGH)'
    )
    parser.addini(
        'bandit_output_format',
        type='args',
        default='screen',
        help='(CURRENTLY NOT IMPLEMENTED) Specify output format'
    )
    parser.addini(
        'bandit_msg_template',
        type='args',
        help='(CURRENTLY NOT IMPLEMENTED) Specify output message template (only usable with --format custom), \
              see CUSTOM FORMAT section for list of available values'
    )
    parser.addini(
        'bandit_verbose',
        type='bool',
        help='Output extra information like excluded and included files'
    )
    parser.addini(
        'bandit_debug',
        type='bool',
        help='Turn on debug mode'
    )
    parser.addini(
        'bandit_ignore_nosec',
        type='bool',
        help='Do not skip lines with # nosec comments'
    )
    parser.addini(
        'bandit_exclude',
        type='args',
        help='Comma-separated list of paths to exclude from scan'
    )
    parser.addini(
        'bandit_baseline',
        type='args',
        help='Path of baseline report to compare against (only JSON-formatted files are accepted)'
    )
    parser.addini(
        'bandit_ini',
        type='args',
        help='Path to a .bandit file that supplies command line arguments'
    )


def pytest_sessionfinish(session, exitstatus):
    if exitstatus == 0:
        return BanditItem(session).runtest()
